@@ -73,6 +73,8 @@
 #'   the color "black" defined the text zone. When a list of masks is given, the
 #'   mask_group aesthetic defines which mask is going to be used. Default to
 #'   \code{NA}, i.e. no mask.
+#' @param show_boxes display the bounding boxes used in the placement algorithm is set
+#' to \code{TRUE}. Default to \code{FALSE}.
 #'
 #' @return a ggplot
 #'
@@ -114,7 +116,8 @@ geom_text_wordcloud <- function(mapping = NULL, data = NULL,
                                 area_corr_power = 1 / .7,
                                 na.rm = FALSE,
                                 show.legend = FALSE,
-                                inherit.aes = TRUE) {
+                                inherit.aes = TRUE,
+                                show_boxes = FALSE) {
   if (!missing(nudge_x) || !missing(nudge_y)) {
     if (!missing(position)) {
       stop("You must specify either `position` or `nudge_x`/`nudge_y`.", call. = FALSE)
@@ -166,6 +169,7 @@ geom_text_wordcloud <- function(mapping = NULL, data = NULL,
       mask = mask,
       area_corr = area_corr,
       area_corr_power = area_corr_power,
+      show_boxes = show_boxes,
       ...
     )
   )
@@ -197,7 +201,8 @@ geom_text_wordcloud_area <- function(mapping = NULL, data = NULL,
                                      area_corr_power = 1 / .7,
                                      na.rm = FALSE,
                                      show.legend = FALSE,
-                                     inherit.aes = TRUE) {
+                                     inherit.aes = TRUE,
+                                     show_boxes = FALSE) {
   if (!missing(nudge_x) || !missing(nudge_y)) {
     if (!missing(position)) {
       stop("You must specify either `position` or `nudge_x`/`nudge_y`.", call. = FALSE)
@@ -249,6 +254,7 @@ geom_text_wordcloud_area <- function(mapping = NULL, data = NULL,
       mask = mask,
       area_corr = area_corr,
       area_corr_power = area_corr_power,
+      show_boxes = show_boxes,
       ...
     )
   )
@@ -256,15 +262,15 @@ geom_text_wordcloud_area <- function(mapping = NULL, data = NULL,
 
 
 GeomTextWordcloud <- ggproto("GeomTextWordcloud", Geom,
-  required_aes = c("label"),
+                             required_aes = c("label"),
 
-  default_aes = aes(
-    x = 0.5, y = 0.5,
-    colour = "black", size = 3.88, angle = 0, hjust = 0.5,
-    vjust = 0.5, alpha = NA, family = "", fontface = 1, lineheight = 1.2, mask_group = 1L, angle_group = 1L
-  ),
+                             default_aes = aes(
+                               x = 0.5, y = 0.5,
+                               colour = "black", size = 3.88, angle = 0, hjust = 0.5,
+                               vjust = 0.5, alpha = NA, family = "", fontface = 1, lineheight = 1.2, mask_group = 1L, angle_group = 1L
+                             ),
 
-  setup_data = function(data, params) {
+                             setup_data = function(data, params) {
     if (params$area_corr) {
       dev_inch <- dev.size("in")
       dev_pix <- dev.size("px")
@@ -294,23 +300,24 @@ GeomTextWordcloud <- ggproto("GeomTextWordcloud", Geom,
   },
 
   draw_panel = function(data, panel_params, coord,
-                          parse = FALSE,
-                          eccentricity = 0.65,
-                          rstep = .01,
-                          tstep = .02,
-                          perc_step = .01,
-                          max_steps = 10,
-                          grid_size = 4,
-                          max_grid_size = 128,
-                          grid_margin = 1,
-                          xlim = c(NA, NA),
-                          ylim = c(NA, NA),
-                          seed = NA,
-                          rm_outside = FALSE,
-                          shape = "circle",
-                          mask = NA,
-                          area_corr = FALSE,
-                          area_corr_power = 1 / .7) {
+                        parse = FALSE,
+                        eccentricity = 0.65,
+                        rstep = .01,
+                        tstep = .02,
+                        perc_step = .01,
+                        max_steps = 10,
+                        grid_size = 4,
+                        max_grid_size = 128,
+                        grid_margin = 1,
+                        xlim = c(NA, NA),
+                        ylim = c(NA, NA),
+                        seed = NA,
+                        rm_outside = FALSE,
+                        shape = "circle",
+                        mask = NA,
+                        area_corr = FALSE,
+                        area_corr_power = 1 / .7,
+                        show_boxes = FALSE) {
     lab <- data$label
     if (parse) {
       lab <- parse_safe(as.character(lab))
@@ -350,6 +357,7 @@ GeomTextWordcloud <- ggproto("GeomTextWordcloud", Geom,
       mask = mask,
       area_corr = area_corr,
       area_corr_power = area_corr_power,
+      show_boxes = show_boxes,
       cl = "textwordcloudtree",
       name = "geom_text_wordcloud"
     )
@@ -474,6 +482,11 @@ makeContent.textwordcloudtree <- function(x) {
     shape = x$shape
   )
 
+  if (x$show_boxes) {
+    lapply(seq_along(boxes_text), make_boxgrob,
+           boxes_text, boxes, wordcloud)
+  }
+
   grobs <- lapply(seq_along(valid_strings), make_textgrob, x, valid_strings, wordcloud)
   class(grobs) <- "gList"
 
@@ -501,9 +514,14 @@ just_dir <- function(x, tol = 0.001) {
   out
 }
 
-compute_mask <- function(tg_inch, gw_pix, gh_pix, dev_dpi, f_mask, Cairo_fix = TRUE) {
+compute_mask <- function(tg_inch, gw_pix, gh_pix, dev_dpi, f_mask) {
   prev_dev_id <- dev.cur()
-  dev_id <- Cairo(width = gw_pix, height = gh_pix, dpi = dev_dpi, units = "px", type = "raster")
+  tmp_file <- tempfile(fileext = "_wordcloud.png")
+  png(
+    filename = tmp_file,
+    width = gw_pix, height = gh_pix, res = dev_dpi,
+    units = "px"
+  )
   pushViewport(viewport(
     width = gw_pix / dev_dpi,
     height = gh_pix / dev_dpi,
@@ -511,50 +529,29 @@ compute_mask <- function(tg_inch, gw_pix, gh_pix, dev_dpi, f_mask, Cairo_fix = T
   ))
   grid.draw(tg_inch)
   popViewport()
-  img <- grid.cap()
   dev.off()
   dev.set(prev_dev_id)
-  if (Cairo_fix) {
-    # UTF8 character are not always rendered by the Cairo device
-    # Try png
-    if (all(img == "transparent")) {
-      prev_dev_id <- dev.cur()
-      tmp_file <- tempfile(fileext = "png")
-      png(
-        filename = tmp_file,
-        width = gw_pix, height = gh_pix, res = dev_dpi,
-        units = "px"
-      )
-      pushViewport(viewport(
-        width = gw_pix / dev_dpi,
-        height = gh_pix / dev_dpi,
-        default.units = "inch"
-      ))
-      grid.draw(tg_inch)
-      popViewport()
-      dev.off()
-      dev.set(prev_dev_id)
-      tmp_png <- readPNG(tmp_file)
-      file.remove(tmp_file)
-      img <- apply(tmp_png, c(1, 2), sum)
-      img[which(img == 3)] <- "transparent"
-    }
-    # Fallback to a rectangle
-    if (all(img == "transparent")) {
-      rot <- tg_inch$rot
-      tg_inch$rot <- 0
-      w_inch <- convertWidth(grobWidth(tg_inch), "inch", TRUE)
-      h_inch <- convertHeight(grobHeight(tg_inch), "inch", TRUE)
-      desc_inch <- convertHeight(grobDescent(tg_inch), "inch", TRUE)
-      prev_dev_id <- dev.cur()
-      dev_id <- Cairo(width = gw_pix, height = gh_pix, dpi = dev_dpi, units = "px", type = "raster")
-      pushViewport(viewport(
-        width = gw_pix / dev_dpi,
-        height = gh_pix / dev_dpi,
-        default.units = "inch"
-      ))
-      pushViewport(viewport(
-        x = gw_pix / 2 / dev_dpi,
+  tmp_png <- readPNG(tmp_file)
+  file.remove(tmp_file)
+  img <- f_mask(tmp_png)
+
+  # Fallback to a rectangle
+  if (!any(img)) {
+    rot <- tg_inch$rot
+    tg_inch$rot <- 0
+    w_inch <- convertWidth(grobWidth(tg_inch), "inch", TRUE)
+    h_inch <- convertHeight(grobHeight(tg_inch), "inch", TRUE)
+    asc_inch <- convertHeight(grobAscent(tg_inch), "inch", TRUE)
+    desc_inch <- convertHeight(grobDescent(tg_inch), "inch", TRUE)
+    prev_dev_id <- dev.cur()
+    tmp_file <- tempfile(fileext = "png")
+    png(
+      filename = tmp_file,
+      width = gw_pix, height = gh_pix, res = dev_dpi,
+      units = "px"
+    )
+    pushViewport(viewport(
+      x = gw_pix / 2 / dev_dpi,
         y = gh_pix / 2 / dev_dpi,
         width = gw_pix / dev_dpi,
         height = gh_pix / dev_dpi,
@@ -562,19 +559,19 @@ compute_mask <- function(tg_inch, gw_pix, gh_pix, dev_dpi, f_mask, Cairo_fix = T
         angle = rot
       ))
       grid.rect(
-        x = gw_pix / 2 / dev_dpi, gh_pix / 2 / dev_dpi - desc_inch / 2,
-        width = w_inch, height = h_inch + desc_inch,
+        x = gw_pix / 2 / dev_dpi,
+        y = gh_pix / 2 / dev_dpi  + (asc_inch -h_inch) / 2 - desc_inch / 2,
+        width = w_inch, height = asc_inch + desc_inch,
         default.units = "inch",
         gp = gpar(fill = "black")
       )
       popViewport()
-      popViewport()
-      img <- grid.cap()
       dev.off()
       dev.set(prev_dev_id)
-    }
+      tmp_png <- readPNG(tmp_file)
+      file.remove(tmp_file)
   }
-  f_mask(img)
+  f_mask(tmp_png)
 }
 
 compute_newsize <- function(i, data, dev_dpi, area_corr_power) {
@@ -593,7 +590,9 @@ compute_newsize <- function(i, data, dev_dpi, area_corr_power) {
   )
 
   gw_inch <- convertWidth(grobWidth(tg_inch), "inch", TRUE) * 1.2
-  gh_inch <- convertHeight(grobHeight(tg_inch), "inch", TRUE) * 1.2 + 2 * convertHeight(grobDescent(tg_inch), "inch", TRUE)
+  gh_inch <- convertHeight(grobAscent(tg_inch), "inch", TRUE) * 1.2 +
+    convertHeight(grobHeight(tg_inch), "inch", TRUE) * 1.2 +
+    convertHeight(grobDescent(tg_inch), "inch", TRUE) * 1.2
 
   gw_pix <- max(1, ceiling(gw_inch * dev_dpi))
   gh_pix <- max(1, ceiling(gh_inch * dev_dpi))
@@ -611,7 +610,8 @@ compute_newsize <- function(i, data, dev_dpi, area_corr_power) {
   )
 
   # Compute the text mask
-  mask <- compute_mask(tg_inch, gw_pix, gh_pix, dev_dpi, function(img) (img != "transparent"))
+  mask <- compute_mask(tg_inch, gw_pix, gh_pix, dev_dpi,
+                       function(img) { apply(img, c(1,2), sum) != 3 })
   area <- sum(mask)
   if (area > 0) {
     row$size^(area_corr_power) / sqrt(area)
@@ -639,10 +639,7 @@ compute_mask_boxes <- function(mask_matrix, dev_dpi, grid_size, max_grid_size, g
   # Compute the mask mask
   mask <- compute_mask(
     mask_raster, gw_pix, gh_pix, dev_dpi,
-    function(img) {
-      img != "black"
-    },
-    Cairo_fix = FALSE
+    function(img) { apply(img, c(1,2), sum) != 0 }
   )
 
   compute_boxes_from_mask(
@@ -673,8 +670,13 @@ compute_text_boxes <- function(i, x, dev_dpi, grid_size, max_grid_size, grid_mar
     )
   )
 
-  gw_inch <- convertWidth(grobWidth(tg_inch), "inch", TRUE) * 1.2
-  gh_inch <- convertHeight(grobHeight(tg_inch), "inch", TRUE) * 1.2 + 2 * convertHeight(grobDescent(tg_inch), "inch", TRUE)
+  gw_inch_ <- convertWidth(grobWidth(tg_inch), "inch", TRUE) * 1.2
+  gh_inch_ <- convertHeight(grobAscent(tg_inch), "inch", TRUE) * 1.2 +
+    convertHeight(grobHeight(tg_inch), "inch", TRUE) * 1.2 +
+    convertHeight(grobDescent(tg_inch), "inch", TRUE) * 1.2
+
+  gw_inch <- gw_inch_
+  gh_inch <- gh_inch_
 
   gw_pix <- max(1, ceiling(gw_inch * dev_dpi / grid_size)) * grid_size
   gh_pix <- max(1, ceiling(gh_inch * dev_dpi / grid_size)) * grid_size
@@ -696,9 +698,7 @@ compute_text_boxes <- function(i, x, dev_dpi, grid_size, max_grid_size, grid_mar
   # Compute the text mask
   mask <- compute_mask(
     tg_inch, gw_pix, gh_pix, dev_dpi,
-    function(img) {
-      img != "transparent"
-    }
+    function(img) { apply(img, c(1,2), sum) != 3 }
   )
 
   compute_boxes_from_mask(
@@ -765,7 +765,9 @@ compute_boxes_from_mask <- function(mask, gw_pix, gh_pix, gw_ratio, gh_ratio, gr
       mask_lists <- rbind(mask_lists, mask_list)
     }
 
-    cur_mask <- next_mask
+    if (st != max(step)) {
+      cur_mask <- next_mask
+    }
   }
 
   mask_lists
@@ -774,7 +776,7 @@ compute_boxes_from_mask <- function(mask, gw_pix, gh_pix, gw_ratio, gh_ratio, gr
 make_textgrob <- function(i, x, valid_strings, wordcloud) {
   xi <- valid_strings[i]
   row <- x$data[xi, , drop = FALSE]
-  # browser()
+
   textGrob(
     x$lab[xi],
     # Position of text bounding boxes.
@@ -792,3 +794,16 @@ make_textgrob <- function(i, x, valid_strings, wordcloud) {
     vjust = x$data$vjust[i]
   )
 }
+
+make_boxgrob <- function
+(i, boxes_text, boxes, wordcloud) {
+  grid.rect(
+    x = unit(wordcloud$x[boxes_text[i]+1] +
+               (boxes[i,1]+boxes[i,3])/2, "native"),
+    y = unit(wordcloud$y[boxes_text[i]+1] +
+               (boxes[i,2]+boxes[i,4])/2, "native"),
+    width = unit(boxes[i,3] - boxes[i,1], "native"),
+    height = unit(boxes[i,4] - boxes[i,2], "native"),
+    gp = gpar(col = alpha("red", .5))
+  )
+}
